@@ -25,15 +25,15 @@ Enables identical use for installed and uninstalled versions.
 
 import os.path
 
-LIBDIR = '/usr/local/lib'
-PKGDATADIR = '/usr/local/share/pitivi'
-PIXMAPDIR = '/usr/local/share/pixmaps'
-pitivi_version = '0.13.2'
+LIBDIR = '/usr/lib'
+PKGDATADIR = '/usr/share/pitivi'
+PIXMAPDIR = '/usr/share/pixmaps'
+pitivi_version = '0.13.2.2'
 APPNAME = 'PiTiVi'
 PYGTK_REQ = '2.14.0'
 PYGST_REQ = '0.10.16'
 GST_REQ = '0.10.24'
-GNONLIN_REQ = '0.10.11.3'
+GNONLIN_REQ = '0.10.12.2'
 PYCAIRO_REQ = '1.0.0'
 
 def _get_root_dir():
