@@ -89,6 +89,8 @@ class SingleDecodeBin(gst.Bin):
 
         self.debug("stream:%r" % self.stream)
 
+        self.pending_newsegment = False
+        self.eventProbeId = None
 
     ## internal methods
 
@@ -282,13 +284,64 @@ class SingleDecodeBin(gst.Bin):
         if self._srcpad:
             return
         self._markValidElements(element)
-        self._removeUnusedElements(self.typefind)
-        self.log("ghosting pad %s" % pad.get_name())
+        gobject.idle_add(self._removeUnusedElements, self.typefind)
+        if pad.props.caps is not None:
+            caps = pad.props.caps
+        else:
+            caps = pad.get_caps()
+
         self._srcpad = gst.GhostPad("src", pad)
         self._srcpad.set_active(True)
+
+        if caps.is_fixed():
+            self._exposePad(target=pad)
+        else:
+            self._blockPad(target=pad)
+
+    def _exposePad(self, target):
+        self.log("ghosting pad %s" % target.get_name())
         self.add_pad(self._srcpad)
         self.post_message(gst.message_new_state_dirty(self))
 
+    def _blockPad(self, target):
+        # don't pass target as an argument to set_blocked_async. Avoids
+        # triggering a bug in gst-python where pad_block_destroy_data calls
+        # CPython without acquiring the GIL
+        self._target = target
+        self._eventProbeId = target.add_event_probe(self._padEventCb)
+        self._srcpad.set_blocked_async(True, self._padBlockedCb)
+
+    def _unblockPad(self, target):
+        target.remove_event_probe(self._eventProbeId)
+        self._eventProbeId = None
+        self._srcpad.set_blocked_async(False, self._padBlockedCb)
+
+    def _padBlockedCb(self, ghost, blocked):
+        if not blocked:
+            if self.pending_newsegment is not None:
+                self._srcpad.push_event(self.pending_newsegment)
+                self.pending_newsegment = None
+            return
+
+        self._exposePad(target=self._target)
+        self._unblockPad(target=self._target)
+
+    def _padEventCb(self, pad, event):
+        if event.type == gst.EVENT_TAG:
+            self.debug("dropping TAG event")
+            return False
+
+        if event.type != gst.EVENT_NEWSEGMENT:
+            self.warning("first event: %s is not a NEWSEGMENT, bailing out" %
+                    event)
+            self._exposePad(target=pad)
+            self._unblockPad(target=pad)
+            return True
+
+        self.debug("stored pending newsegment")
+        self.pending_newsegment = event
+        return False
+
     def _markValidElements(self, element):
         """
         Mark this element and upstreams as valid
@@ -310,18 +363,20 @@ class SingleDecodeBin(gst.Bin):
         for pad in element.src_pads():
             if pad.is_linked():
                 peer = pad.get_peer().get_parent()
-                self._removeUnusedElements(peer)
-                if not peer in self._validelements:
-                    self.log("removing %s" % peer.get_name())
-                    pad.unlink(pad.get_peer())
-                    peer.set_state(gst.STATE_NULL)
-                    self.remove(peer)
+                if isinstance(peer, gst.Element):
+                    self._removeUnusedElements(peer)
+                    if not peer in self._validelements:
+                        self.log("removing %s" % peer.get_name())
+                        pad.unlink(pad.get_peer())
+                        peer.set_state(gst.STATE_NULL)
+                        self.remove(peer)
 
     def _cleanUp(self):
         self.log("")
         if self._srcpad:
             self.remove_pad(self._srcpad)
         self._srcpad = None
+        self._target = None
         for element in self._validelements:
             element.set_state(gst.STATE_NULL)
             self.remove(element)
