@@ -20,7 +20,6 @@
 # Boston, MA 02111-1307, USA.
 
 import gst
-import weakref
 
 from pitivi.signalinterface import Signallable
 from pitivi.utils import get_controllable_properties, getPreviousObject, \
@@ -136,14 +135,19 @@ class Interpolator(Signallable, Loggable):
         'keyframe-moved' : ['keyframe'],
     }
 
-    def __init__(self, trackobject, element, prop):
+    def __init__(self, trackobject, element, prop, minimum=None, maximum=None):
         Loggable.__init__(self)
         self.debug("track:%r, element:%r, property:%r", trackobject, element, prop)
         self._keyframes = []
-        # FIXME: get this from the property's param spec
-        # NOTE: keyframes necessarily work only on a closed range
-        self.lower = 0
-        self.upper = 1
+
+        if minimum is None:
+            minimum = prop.minimum
+        if maximum is None:
+            maximum = prop.maximum
+        assert not ((minimum is None) or (maximum is None))
+        self.lower = minimum
+        self.upper = maximum
+        self.range = maximum - minimum
 
         # FIXME: don't necessarily want to create separate controllers for
         # each Interpolator. We should instead create a ControlSource for each
@@ -176,12 +180,9 @@ class Interpolator(Signallable, Loggable):
         if isinstance(time_or_keyframe, Keyframe):
             keyframe = time_or_keyframe
         else:
-            # TODO: calculate value so that the new point doesn't change the shape
-            # of the curve when added. This might be tricky to achieve with cubic
-            # interpolation, but should work fine for linear and step
-            # interpolation.
             if value is None:
-                value = self._default
+                value = self._controller.get(self._property.name,
+                    time_or_keyframe)
             if mode is None:
                 # FIXME: Controller.get_interpolation_mode is not wrapped in
                 # gst-python, so for now we assume the default is linear.
@@ -325,19 +326,25 @@ class TrackObject(Signallable, Loggable):
 
         self._rebuild_interpolators = False
 
-        factory_properties = self.factory.getInterpolatedProperties(self.stream).keys()
+        factory_properties = self.factory.getInterpolatedProperties(self.stream)
 
         old_interpolators = self.interpolators
         self.interpolators = {}
         for gst_object, gst_object_property in \
                 get_controllable_properties(self.gnl_object):
-            if gst_object_property.name not in factory_properties:
+            prop_name = gst_object_property.name
+            if prop_name not in factory_properties:
                 continue
 
             try:
-                interpolator = old_interpolators[gst_object_property.name][1]
+                interpolator = old_interpolators[prop_name][1]
             except KeyError:
-                interpolator = Interpolator(self, gst_object, gst_object_property)
+                if factory_properties[prop_name]:
+                    lower, upper = factory_properties[prop_name]
+                else:
+                    lower, upper = None, None
+                interpolator = Interpolator(self, gst_object,
+                    gst_object_property, lower, upper)
             else:
                 interpolator.attachToElementProperty(gst_object_property,
                         gst_object)
@@ -664,9 +671,12 @@ class Track(Signallable):
             self.default_track_object = None
             raise
 
+    def _skipDefaultTrackObject(self, timeline_object):
+        return timeline_object is self.default_track_object
+
     def getPreviousTrackObject(self, obj, priority=-1):
         prev = getPreviousObject(obj, self.track_objects, priority,
-                [self.default_track_object])
+                self._skipDefaultTrackObject)
         if prev is None:
             raise TrackError("no previous track object", obj)
 
@@ -674,7 +684,7 @@ class Track(Signallable):
 
     def getNextTrackObject(self, obj, priority=-1):
         next = getNextObject(obj, self.track_objects, priority,
-                [self.default_track_object])
+                self._skipDefaultTrackObject)
         if next is None:
             raise TrackError("no next track object", obj)
 
@@ -719,7 +729,7 @@ class Track(Signallable):
             raise TrackError()
         track_object.makeBin()
 
-        track_object.track = weakref.proxy(self)
+        track_object.track = self
 
         start_insort_right(self.track_objects, track_object)
 
