@@ -45,7 +45,8 @@ KW_WIDTH = 7
 KW_HEIGHT = 7
 KW_WIDTH2 = KW_WIDTH / 2
 KW_HEIGHT2 = KW_HEIGHT / 2
-
+KW_MOUSE_WIDTH = KW_WIDTH2 + 1
+KW_MOUSE_HEIGHT = KW_HEIGHT2 + 1
 CURVE_STROKE_WIDTH = 2.0
 HAND = gtk.gdk.Cursor(gtk.gdk.HAND2)
 
@@ -56,11 +57,11 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
     class Controller(Controller):
 
         _cursor = HAND
+        _kf = None
 
         def drag_start(self, item, target, event):
             self._view.app.action_log.begin("volume change")
             initial = self.from_item_event(item, event)
-            self._kf = self._view.findKeyframe(initial)
             if self._kf:
                 self._mousedown = self._view.keyframes[self._kf] - initial
             if not self._kf:
@@ -71,7 +72,6 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
                     self.xyToTimeValue(initial)[0])
 
         def drag_end(self, item, target, event):
-            self._kf = None
             self._view.app.action_log.commit()
 
         def set_pos(self, obj, pos):
@@ -86,13 +86,19 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
                         - self.pos(self._view))
                     kf.value = value
 
+        def hover(self, item, target, event):
+            coords = self.from_item_event(item, event)
+            self._kf = self._view.findKeyframe(coords)
+            self._view.setFocusedKf(self._kf)
+
         def double_click(self, pos):
             interpolator = self._view.interpolator
             kf = self._view.findKeyframe(pos)
             if kf is None:
                 time, value = self.xyToTimeValue(pos)
                 self._view.app.action_log.begin("add volume point")
-                interpolator.newKeyframe(time)
+                kf = interpolator.newKeyframe(time)
+                self._view.setFocusedKf(kf)
                 self._view.app.action_log.commit()
             else:
                 self._view.app.action_log.begin("remove volume point")
@@ -110,6 +116,9 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
             return time, value
 
         def enter(self, item ,target):
+            coords = self.from_item_event(item, self._last_event)
+            self._kf = self._view.findKeyframe(coords)
+            self._view.setFocusedKf(self._kf)
             self._view.focus()
 
         def leave(self, item, target):
@@ -126,6 +135,7 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
         self.element = element
         self.props.pointer_events = goocanvas.EVENTS_STROKE
         self.interpolator = interpolator
+        self._focused_kf = None
         self.normal()
 
 ## properties
@@ -159,6 +169,8 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
     def keyframeRemoved(self, unused_interpolator, keyframe):
         if keyframe in self.keyframes:
             del self.keyframes[keyframe]
+            if keyframe is self._focused_kf:
+                self._focused_kf = None
         self.changed(False)
 
     @handler(interpolator, "keyframe-added")
@@ -210,6 +222,13 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
             cr.fill_preserve()
             cr.set_source_rgb(1, 0, 0)
             cr.stroke()
+            # re-draw the focused keyframe, if it exists, inverted
+            if self._focused_kf:
+                self._controlPoint(cr, self._focused_kf)
+                cr.set_source_rgb(1, 0, 0)
+                cr.fill_preserve()
+                cr.set_source_rgb(1, 1, 1)
+                cr.stroke()
 
     def make_curve(self, cr):
         if not self.interpolator:
@@ -236,20 +255,26 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
 
 ## public
 
+    def setFocusedKf(self, kf):
+        if self._focused_kf is not kf:
+            self.changed(False)
+        self._focused_kf = kf
+
     def focus(self):
         self.line_width = CURVE_STROKE_WIDTH * 1.5
         self.changed(False)
 
     def normal(self):
         self.line_width = CURVE_STROKE_WIDTH
+        self.setFocusedKf(None)
         self.changed(False)
 
     def findKeyframe(self, pos):
         x, y = pos
         for keyframe, value in self.keyframes.iteritems():
             kx, ky = value
-            if (between(kx - KW_WIDTH2, x, kx + KW_HEIGHT2) and
-                between(ky - KW_HEIGHT2, y, ky + KW_HEIGHT2)):
+            if (between(kx - KW_MOUSE_WIDTH, x, kx + KW_MOUSE_WIDTH) and
+                between(ky - KW_MOUSE_HEIGHT, y, ky + KW_MOUSE_HEIGHT)):
                 return keyframe
         return None
 
