@@ -42,9 +42,9 @@ from gettext import gettext as _
 
 from pitivi.log.loggable import Loggable
 
-from timeline import Timeline
-from projecttabs import ProjectTabs
-from viewer import PitiviViewer
+from pitivi.ui.timeline import Timeline
+from pitivi.ui.projecttabs import ProjectTabs
+from pitivi.ui.viewer import PitiviViewer
 from pitivi.configure import pitivi_version, APPNAME, get_pixmap_dir, \
      get_global_pixmap_dir
 from pitivi.ui import dnd
@@ -54,6 +54,10 @@ from pitivi.settings import GlobalSettings
 from pitivi.receiver import receiver, handler
 import pitivi.formatters.format as formatter
 from pitivi.sourcelist import SourceListError
+from pitivi.ui.sourcelist import SourceList
+from pitivi.ui.common import beautify_factory
+from pitivi.utils import beautify_length
+from pitivi.ui.zoominterface import Zoomable
 
 if HAVE_GCONF:
     D_G_INTERFACE = "/desktop/gnome/interface"
@@ -354,7 +358,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _createUi(self, instance):
         """ Create the graphical interface """
         self.set_title("%s v%s" % (APPNAME, pitivi_version))
-        self.set_geometry_hints(min_width=800, min_height=480)
         self.connect("delete-event", self._deleteCb)
         self.connect("configure-event", self._configureCb)
 
@@ -375,7 +378,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
         vpaned.pack2(self.timeline, resize=False, shrink=False)
         hpaned = gtk.HPaned()
         vpaned.pack1(hpaned, resize=True, shrink=False)
-        self.projecttabs = ProjectTabs(instance, self.uimanager)
+        self.projecttabs = ProjectTabs()
+
+        self.sourcelist = SourceList(instance, self.uimanager)
+        self.projecttabs.append_page(self.sourcelist, gtk.Label(_("Clip Library")))
         self._connectToSourceList()
 
         hpaned.pack1(self.projecttabs, resize=True, shrink=False)
@@ -418,7 +424,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         # toolbar buttons should be clickable with the mouse cursor at the
         # very bottom of the screen.
         ttb = self.uimanager.get_widget("/TimelineToolBar")
-        vbox.pack_start(ttb, False)
+        vbox.pack_start(ttb, expand=False)
 
         self.show_all()
 
@@ -435,8 +441,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _connectToSourceList(self):
         # FIXME: projecttabs creates the "components" but then has no API to get
         # them back
-        sourcelist = self.projecttabs._full_list[0]
-        sourcelist.connect('play', self._sourceListPlayCb)
+        self.sourcelist.connect('play', self._sourceListPlayCb)
 
     def toggleFullScreen(self):
         """ Toggle the fullscreen mode of the application """
@@ -683,6 +688,40 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self._connectToProjectSources(project.sources)
         can_render = project.timeline.duration > 0
         self.render_button.set_sensitive(can_render)
+        self._syncDoUndo(self.app.action_log)
+
+        if project.timeline.duration != 0:
+            self._setBestZoomRatio()
+
+    def _setBestZoomRatio(self):
+        # reset the default zoom ratio
+        Zoomable.setZoomRatio(2)
+
+        ruler_width = self.timeline.ruler.get_allocation()[2]
+        while True:
+            timeline_width = Zoomable.nsToPixel(self.project.timeline.duration)
+
+            if ruler_width == timeline_width:
+                # perfect
+                break
+
+            current_level = Zoomable.getCurrentZoomLevel()
+
+            if ruler_width > timeline_width:
+                if current_level == Zoomable.zoom_levels[-1]:
+                    break
+
+                Zoomable.zoomIn()
+                timeline_width = Zoomable.nsToPixel(self.project.timeline.duration)
+                if timeline_width > ruler_width:
+                    Zoomable.zoomOut()
+                    break
+
+            else:
+                if current_level == Zoomable.zoom_levels[0]:
+                    break
+
+                Zoomable.zoomOut()
 
     def _projectManagerNewProjectLoadingCb(self, projectManager, uri):
         self.log("A NEW project is being loaded, deactivate UI")
@@ -780,7 +819,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.destroy()
         self.set_sensitive(True)
 
-    def _projectManagerMissingUriCb(self, instance, formatter, uri):
+    def _projectManagerMissingUriCb(self, instance, formatter, uri, factory):
         dialog = gtk.Dialog(_("Locate missing file..."),
             self,
             gtk.DIALOG_MODAL,
@@ -789,7 +828,14 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.set_border_width(12)
         dialog.get_content_area().set_spacing(6)
 
-        label = gtk.Label(_("Please locate the missing file, '%s'" % uri))
+        text = _("The following file has moved," +
+            " please tell PiTiVi where to find it.") + "\n\n" + \
+            beautify_factory(factory) + "\n" +\
+            _("<b>Duration:</b>") + beautify_length(factory.duration)
+
+        label = gtk.Label()
+        label.set_markup(text)
+        label.set_justify(gtk.JUSTIFY_CENTER)
         dialog.get_content_area().pack_start(label, False, False)
         label.show()
 
@@ -851,6 +897,14 @@ class PitiviMainWindow(gtk.Window, Loggable):
         can_redo = bool(action_log.redo_stacks)
         redo_action.set_sensitive(can_redo)
 
+        if self.project is not None:
+            app_name = "%s %s" % (APPNAME, pitivi_version)
+            title = u"%s \u2014 %s" % (self.project.name, app_name)
+            if dirty:
+                title = "*" + title
+            title = title.encode("utf8")
+            self.set_title(title)
+
 ## PiTiVi current project callbacks
 
     def _setProject(self):
