@@ -27,7 +27,6 @@ import os
 import time
 
 from urllib import unquote
-from xml.sax.saxutils import escape
 from gettext import gettext as _
 from gettext import ngettext
 
@@ -40,7 +39,10 @@ from pitivi.stream import VideoStream, AudioStream, TextStream, \
         MultimediaStream
 from pitivi.settings import GlobalSettings
 from pitivi.utils import beautify_length
+from pitivi.ui.common import beautify_factory, factory_name, \
+    beautify_stream
 from pitivi.log.loggable import Loggable
+from pitivi.sourcelist import SourceListError
 
 GlobalSettings.addConfigSection('clip-library')
 GlobalSettings.addConfigOption('lastImportFolder',
@@ -57,7 +59,8 @@ GlobalSettings.addConfigOption('closeImportDialog',
  COL_INFOTEXT,
  COL_FACTORY,
  COL_URI,
- COL_LENGTH) = range(5)
+ COL_LENGTH,
+ COL_SEARCH_TEXT) = range(6)
 
 ui = '''
 <ui>
@@ -83,45 +86,6 @@ ui = '''
 INVISIBLE = gtk.gdk.pixbuf_new_from_file(os.path.join(get_pixmap_dir(), 
     "invisible.png"))
 
-def beautify_stream(stream):
-
-    if type(stream) == AudioStream:
-        if stream.raw:
-            templ = ngettext("<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
-                    "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
-                    stream.channels)
-            templ = templ % (stream.channels, stream.rate, stream.width)
-            return templ
-
-        return _("<b>Unknown Audio format:</b> %s") % stream.audiotype
-
-    elif type(stream) == VideoStream:
-        if stream.raw:
-            if stream.framerate.num:
-                templ = _("<b>Video:</b> %d x %d <i>pixels</i> at %.2f<i>fps</i>")
-                templ = templ % (stream.par * stream.width , stream.height,
-                        float(stream.framerate))
-            else:
-                templ = _("<b>Image:</b> %d x %d <i>pixels</i>")
-                templ = templ % (stream.par * stream.width, stream.height)
-            return templ
-        return _("<b>Unknown Video format:</b> %s") % stream.videotype
-
-    elif type(stream) == TextStream:
-        return _("<b>Text:</b> %s") % stream.texttype
-
-    raise NotImplementedError
-
-def beautify_factory(factory):
-    ranks = {VideoStream: 0, AudioStream: 1, TextStream: 2, MultimediaStream: 3}
-    def stream_sort_key(stream):
-        return ranks[type(stream)]
-
-    streams = factory.getOutputStreams()
-    streams.sort(key=stream_sort_key)
-    return ("<b>" + escape(unquote(factory.name)) + "</b>\n" +
-        "\n".join((beautify_stream(stream) for stream in streams)))
-
 class SourceList(gtk.VBox, Loggable):
     """ Widget for listing sources """
 
@@ -138,10 +102,8 @@ class SourceList(gtk.VBox, Loggable):
 
         # Store
         # icon, infotext, objectfactory, uri, length
-        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, object, str, str)
-
-        self.set_border_width(5)
-        self.set_spacing(6)
+        self.storemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, object, str, str,
+            str)
 
         # Scrolled Window
         self.scrollwin = gtk.ScrolledWindow()
@@ -180,6 +142,7 @@ class SourceList(gtk.VBox, Loggable):
         self.treeview.connect("row-activated", self._rowActivatedCb)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(False)
+        self.treeview.set_property("search_column", COL_SEARCH_TEXT)
         tsel = self.treeview.get_selection()
         tsel.set_mode(gtk.SELECTION_MULTIPLE)
         tsel.connect("changed", self._treeSelectionChanged)
@@ -468,7 +431,8 @@ class SourceList(gtk.VBox, Loggable):
             beautify_factory(factory),
             factory,
             factory.uri,
-            duration])
+            duration,
+            factory_name(factory)])
         self._displayTreeView()
 
     # sourcelist callbacks
@@ -585,7 +549,10 @@ class SourceList(gtk.VBox, Loggable):
         result = treeview.get_path_at_pos(int(event.x), int(event.y))
         if result:
             path = result[0]
-            return treeview.get_selection().path_is_selected(path)
+            selection = treeview.get_selection()
+
+            return selection.path_is_selected(path) and selection.count_selected_rows() > 1
+
         return False
 
     def _nothingUnderMouse(self, treeview, event):
@@ -703,7 +670,11 @@ class SourceList(gtk.VBox, Loggable):
                 directories = [incoming]
         if directories:
             self.addFolders(directories)
-        self.addUris(filenames)
+        try:
+            self.addUris(filenames)
+        except SourceListError:
+            # filenames already present in the sourcelist
+            pass
 
     def _dndTreeBeginCb(self, unused_widget, context):
         self.info("tree drag_begin")
