@@ -206,9 +206,6 @@ class PitiviViewer(gtk.VBox, Loggable):
 
     def _createUi(self):
         """ Creates the Viewer GUI """
-        self.set_border_width(5)
-        self.set_spacing(5)
-
         # drawing area
         self.aframe = gtk.AspectFrame(xalign=0.5, yalign=0.5, ratio=4.0/3.0,
                                       obey_child=False)
@@ -261,12 +258,19 @@ class PitiviViewer(gtk.VBox, Loggable):
 
         # current time
         self.timelabel = gtk.Label()
-        self.timelabel.set_markup("<tt>00:00:00.000 / --:--:--.---</tt>")
+        self.timelabel.set_markup("<tt>00:00:00.000</tt>")
         self.timelabel.set_alignment(1.0, 0.5)
-        self.timelabel.set_padding(5, 5)
         bbox.pack_start(self.timelabel, expand=False, padding=10)
         self._haveUI = True
 
+        # show the controls and force the aspect frame to have at least the same
+        # width (+110, which is a magic number to minimize dead padding).
+        bbox.show_all()
+        width, height = bbox.size_request()
+        width += 110
+        height = width / self.aframe.props.ratio
+        self.aframe.set_size_request(width , height)
+
     def showControls(self):
         if not self.action:
             return
@@ -364,12 +368,7 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.info("value:%s, frame:%d", gst.TIME_ARGS(value), frame)
         self.current_time = value
         self.current_frame = frame
-        try:
-            duration = self.pipeline.getDuration()
-        except:
-            duration = gst.CLOCK_TIME_NONE
-        self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(value),
-                                                        time_to_string(duration)))
+        self.timelabel.set_markup("<tt>%s</tt>" % time_to_string(value))
         if not self.moving_slider:
             self.posadjust.set_value(float(value))
         return False
@@ -384,9 +383,6 @@ class PitiviViewer(gtk.VBox, Loggable):
             self.posadjust.set_value(float(duration))
         self.posadjust.upper = float(duration)
 
-        self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(self.current_time),
-                                                        time_to_string(duration)))
-
         if duration == 0:
             self._setUiActive(False)
         else:
